from django.core.management.base import CommandError
from django.core.management.commands.migrate import Command as MigrateCommand
from django.db import connections

from isapilib.Connection import create_conn
from isapilib.models import SepaBranch


class Command(MigrateCommand):
    help = 'Migrate from external database'
    exclude_apps = ['admin', 'auth', 'contenttypes', 'oauth2_provider', 'sessions']

    def add_arguments(self, parser):
        parser.add_argument(
            "branch",
            nargs="?",
            help="id branch",
        )
        super().add_arguments(parser)

    def handle(self, *args, **options):
        branch = SepaBranch.objects.get(id=options["branch"])
        connections.databases['external'] = create_conn(branch)
        options['database'] = 'external'

        if not options['app_label']:
            raise CommandError('you must specified app_name')
        elif options['app_label'] in self.exclude_apps:
            raise CommandError(f'You can´t migrate {options["app_label"]} app')

        db_host = connections[options['database']].settings_dict['HOST']
        db_name = connections[options['database']].settings_dict['NAME']
        self.stdout.write(self.style.SUCCESS(f'Running migrations on {db_host} {db_name}:'))
        super().handle(*args, **options)



from django.core.management import BaseCommand
from django.db import connections

from isapilib.Connection import create_conn
from isapilib.models import SepaBranch


class Command(BaseCommand):
    help = 'Check connection to external database'

    def add_arguments(self, parser):
        parser.add_argument(
            "branch",
            nargs="?",
            help="id branch",
        )

    def handle(self, *args, **options):
        branch = SepaBranch.objects.filter(id=options["branch"]).first()

        if branch:
            db_name = f'external-{branch.id}'
            connections.databases[db_name] = create_conn(branch)

            try:
                connection = connections[db_name]
                cursor = connection.cursor()
                cursor.execute('SELECT 1')
                self.stdout.write(self.style.SUCCESS(f'Connection to database {db_name} was successful.'))
            except Exception as e:
                self.stdout.write(self.style.SUCCESS(f'Failed to connect to database {db_name}: {e}'))
        else:
            self.stdout.write(self.style.SUCCESS(f"Branch doesn't exist."))



# Generated by Django 5.0.2 on 2024-07-12 17:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isapilib', '0001_initial'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Agente',
        ),
        migrations.DeleteModel(
            name='Art',
        ),
        migrations.DeleteModel(
            name='Cte',
        ),
        migrations.DeleteModel(
            name='Empresa',
        ),
        migrations.DeleteModel(
            name='Sucursal',
        ),
        migrations.DeleteModel(
            name='TipoOrdenOperacion',
        ),
        migrations.DeleteModel(
            name='Venta',
        ),
        migrations.DeleteModel(
            name='VentaD',
        ),
        migrations.DeleteModel(
            name='Vin',
        ),
        migrations.AddField(
            model_name='apilogs',
            name='status',
            field=models.IntegerField(default=0),
        ),
    ]



class SepaException(Exception):

    def __init__(self, description, user=None, branch=None):
        self.user = user
        self.branch = branch
        super().__init__(description)


class AfectarException(Exception):

    def __init__(self, ok, ok_ref):
        self.ok = ok
        self.ok_ref = ok_ref
        super().__init__(f'{self.ok}: {self.ok_ref}')



import json

from django.db import connections, ProgrammingError

from isapilib.models import ApiLogs


def execute_query(query, sp_params=None, using='default'):
    cursor = connections[using].cursor()

    try:
        cursor.execute(query, sp_params or [])
        results = cursor.fetchall()
    finally:
        cursor.close()

    return results


def declare_variable(name, type, length):
    declaration = f'{name} '

    if type in ('varchar', 'char'):
        declaration += f"{type}({'MAX' if length == -1 else length})"
    else:
        declaration += f'{type}'

    return declaration


def execute_sp(sp_name, sp_params=None, using='default'):
    sp_params = sp_params or []
    sp_output = execute_query('''
        SELECT 
            PARAMETER_NAME, DATA_TYPE, CHARACTER_MAXIMUM_LENGTH
        FROM information_schema.parameters WHERE SPECIFIC_NAME = %s AND PARAMETER_MODE = 'INOUT'
        ''', [sp_name], using=using)

    if isinstance(sp_params, dict):
        parameters = ', '.join([f'@{i}=%s' for i in sp_params.keys()])
    else:
        parameters = ', '.join(['%s' for _ in sp_params])

    sp_call = f'''
        {'SET NOCOUNT ON;' if len(sp_output) > 0 else ''}
        {'DECLARE' if len(sp_output) > 0 else ''} {', '.join([declare_variable(name, type, length) for name, type, length in sp_output])};
        EXEC {sp_name} {parameters}{', ' if len(sp_output) > 0 else ''}{', '.join([f'{name}={name} OUTPUT' for name, _, _ in sp_output])};
        {'SELECT' if len(sp_output) > 0 else ''} {', '.join([name for name, _, _ in sp_output])};
    '''

    cursor = connections[using].cursor()

    try:
        cursor.execute(sp_call, list(sp_params.values()) if isinstance(sp_params, dict) else sp_params or [])
        results = []
        while True:
            try:
                result = cursor.fetchall()
                results.append(result)
                cursor.nextset()
            except ProgrammingError:
                return results[-1] if len(results) > 0 else None
    finally:
        cursor.close()


def execute_fn(fn_name, fn_params=None, using='default'):
    query = f'''SELECT dbo.{fn_name}({', '.join(['%s' for _ in fn_params])})'''
    return execute_query(query, fn_params, using=using)


def get_sucursal(modulo='VTAS', mov='Servicio', sucursal=0, using='default'):
    results = execute_fn("fnCA_GeneraSucursalValida", [modulo, mov, sucursal], using=using)
    return results[0][0]


def get_almacen(modulo='VTAS', mov='Servicio', sucursal=0, using='default'):
    results = execute_fn("fnCA_GeneraAlmacenlValido", [modulo, mov, sucursal], using=using)
    return results[0][0]


def get_uen(modulo='VTAS', mov='Servicio', sucursal=0, concepto='Publico', using='default'):
    results = execute_fn("fnCA_GeneraUENValida", [modulo, mov, sucursal, concepto], using=using)
    return results[0][0]


def get_param_empresa(interfaz, clave, default=None, using='default'):
    results = execute_fn('fnCA_BusquedaClaveParametroInterfazEmpresa', [interfaz, clave], using=using)
    result = results[0][0]
    return result if result is not None and result != '' else default


def get_param_sucursal(sucursal, clave, default=None, using='default'):
    results = execute_fn('fnCA_CatParametrosSucursalValor', [sucursal, clave], using=using)
    result = results[0][0]
    return result if result is not None and result != '' else default


def insert_log(request, response, interfaz, tipo, time=0):
    try:
        response_content = response.content
        log = ApiLogs()
        log.iduser = request.user.pk
        log.tipo = str(tipo or '')
        log.header = str(json.dumps(dict(request.headers)))
        log.request = str(json.dumps(request.data) if isinstance(request.data, (dict, list)) else request.data or '')
        log.response = str(response_content.decode('utf-8') or '')
        log.status = response.status_code if hasattr(response, 'status_code') else 0
        log.url = request.build_absolute_uri()
        log.interfaz = str(interfaz or '')
        log.response_time = time
        log.save()
    except Exception as e:
        print(f'Warning: Failed to save logs', e)


def clean_request_json(request: dict) -> dict:
    cleaned_request = {}
    for k, v in request.items():
        if v is None or v == '':
            continue

        if isinstance(v, dict):
            v = clean_request_json(v)

        cleaned_request[k] = v

    return cleaned_request



# Generated by Django 5.0.2 on 2024-07-12 17:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isapilib', '0002_delete_agente_delete_art_delete_cte_delete_empresa_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='userapi',
            name='is_admin',
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name='apilogs',
            name='status',
            field=models.IntegerField(null=True),
        ),
    ]



from setuptools import setup, find_packages

setup(
    name='isapilib',
    version='1.8.2',
    packages=find_packages(),
    include_package_data=True,
    package_data={
        'templates': ['templates/*'],
        'static': ['static/*'],
    },
    install_requires=[
        'django>=4.2',
        'djangorestframework>=3.0.0',
        'djangorestframework-simplejwt>=5.0.0',
        'Pillow>=9.4.0',
    ],
)
